//------------------------------------------------------------------
#property copyright "www.forex-tsd.com" 
#property link      "www.forex-tsd.com" 
//------------------------------------------------------------------
#property indicator_separate_window
#property indicator_buffers 5
#property indicator_color1  LimeGreen
#property indicator_color2  LimeGreen
#property indicator_color3  PaleVioletRed
#property indicator_color4  PaleVioletRed
#property indicator_color5  DarkGray
#property indicator_width1  2
#property indicator_width3  2
#property indicator_width5  2
#property indicator_minimum -50
#property indicator_maximum +50
#property indicator_levelcolor DarkGray
#property strict
//
//
//
//
//

enum prices
{
   pr_close,   // Close
   pr_open,    // Open
   pr_high,    // High
   pr_low,     // Low
   pr_median,  // Median
   pr_typical, // Typical
   pr_weighted // Weighted
};

extern double RsiPeriod  = 25;       // Rsi period
extern prices RsiPrice   = pr_close; // Price to use
extern double Hot        = 0.7;      // Rsi "hot" field
extern bool   OriginalT3 = false;    // Use Tim Tillson calculation?
extern double LevelUp    =  30;      // Upper level
extern double LevelDown  = -30;      // lower level

double rsi[];
double rsihuu[];
double rsihud[];
double rsihdu[];
double rsihdd[];
double slope[];

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(6);
   SetIndexBuffer(0,rsihuu); SetIndexStyle(0, DRAW_HISTOGRAM);
   SetIndexBuffer(1,rsihud); SetIndexStyle(1, DRAW_HISTOGRAM);
   SetIndexBuffer(2,rsihdd); SetIndexStyle(2, DRAW_HISTOGRAM);
   SetIndexBuffer(3,rsihdu); SetIndexStyle(3, DRAW_HISTOGRAM);
   SetIndexBuffer(4,rsi);
   SetIndexBuffer(5,slope);
      SetLevelValue(0,0);
      SetLevelValue(1,LevelUp);
      SetLevelValue(2,LevelDown);
      IndicatorShortName("rsi t3 ("+DoubleToStr(RsiPeriod,1)+","+DoubleToStr(Hot,2)+")");
   return(0);
}
int deinit() { return(0); }

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars < 0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);

   //
   //
   //
   //
   //
    
   for(int i = limit; i >= 0; i--)
   {
      rsi[i] = iRsi(iMA(NULL,0,1,0,MODE_SMA,(int)RsiPrice,i),RsiPeriod,Hot,OriginalT3,i);
      rsihuu[i] = EMPTY_VALUE;
      rsihud[i] = EMPTY_VALUE;
      rsihdd[i] = EMPTY_VALUE;
      rsihdu[i] = EMPTY_VALUE;

      //
      //
      //
      //
      //
                           
      if (i<Bars-1)
      {
         slope[i] = slope[i+1];
            if (rsi[i] > rsi[i+1]) slope[i] =  1;
            if (rsi[i] < rsi[i+1]) slope[i] = -1;
            if (rsi[i]>0)
            if (slope[i]==1)
                  rsihuu[i] = rsi[i];
            else  rsihud[i] = rsi[i];
            if (rsi[i]<0)
            if (slope[i]==1)
                  rsihdu[i] = rsi[i];
            else  rsihdd[i] = rsi[i];
      }            
   }      
   return(0);
}


//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double workRsi[][1];
#define _price  0

double iRsi(double price, double period, double hot, bool original, int i)
{
   if (ArrayRange(workRsi,0)!=Bars) ArrayResize(workRsi,Bars); int r = Bars-i-1; workRsi[r][_price] = price;
   
   //
   //
   //
   //
   //
   
   if (r>0)
   {
      double chng   = workRsi[r][_price]-workRsi[r-1][_price];
      double change = iT3(        chng ,period,hot,original,i,0);
      double changa = iT3(MathAbs(chng),period,hot,original,i,1);
      if (changa != 0)
            return(MathMin(MathMax(50.0*(change/MathMax(changa,0.0000001)),-50),50));
      else  return(0);
   }
   else return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double workT3[][18];
double workT3Coeffs[][6];
#define _period 0
#define _c1     1
#define _c2     2
#define _c3     3
#define _c4     4
#define _alpha  5

//
//
//
//
//

double iT3(double price, double period, double hot, bool original, int i, int instanceNo=0)
{
   if (ArrayRange(workT3,0) != Bars)                ArrayResize(workT3,Bars);
   if (ArrayRange(workT3Coeffs,0) < (instanceNo+1)) ArrayResize(workT3Coeffs,instanceNo+1);

   if (workT3Coeffs[instanceNo][_period] != period)
   {
     workT3Coeffs[instanceNo][_period] = period;
        double a = hot;
            workT3Coeffs[instanceNo][_c1] = -a*a*a;
            workT3Coeffs[instanceNo][_c2] = 3*a*a+3*a*a*a;
            workT3Coeffs[instanceNo][_c3] = -6*a*a-3*a-3*a*a*a;
            workT3Coeffs[instanceNo][_c4] = 1+3*a+a*a*a+3*a*a;
            if (original)
                 workT3Coeffs[instanceNo][_alpha] = 2.0/(1.0 + period);
            else workT3Coeffs[instanceNo][_alpha] = 2.0/(2.0 + (period-1.0)/2.0);
   }
   
   //
   //
   //
   //
   //
   
   int buffer = instanceNo*6;
   int r = Bars-i-1;
   if (r == 0)
      {
         workT3[r][0+buffer] = price;
         workT3[r][1+buffer] = price;
         workT3[r][2+buffer] = price;
         workT3[r][3+buffer] = price;
         workT3[r][4+buffer] = price;
         workT3[r][5+buffer] = price;
      }
   else
      {
         workT3[r][0+buffer] = workT3[r-1][0+buffer]+workT3Coeffs[instanceNo][_alpha]*(price              -workT3[r-1][0+buffer]);
         workT3[r][1+buffer] = workT3[r-1][1+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][0+buffer]-workT3[r-1][1+buffer]);
         workT3[r][2+buffer] = workT3[r-1][2+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][1+buffer]-workT3[r-1][2+buffer]);
         workT3[r][3+buffer] = workT3[r-1][3+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][2+buffer]-workT3[r-1][3+buffer]);
         workT3[r][4+buffer] = workT3[r-1][4+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][3+buffer]-workT3[r-1][4+buffer]);
         workT3[r][5+buffer] = workT3[r-1][5+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][4+buffer]-workT3[r-1][5+buffer]);
      }

   //
   //
   //
   //
   //
   
   return(workT3Coeffs[instanceNo][_c1]*workT3[r][5+buffer] + 
          workT3Coeffs[instanceNo][_c2]*workT3[r][4+buffer] + 
          workT3Coeffs[instanceNo][_c3]*workT3[r][3+buffer] + 
          workT3Coeffs[instanceNo][_c4]*workT3[r][2+buffer]);
}